import levels

import pygame
from pygame.locals import *

import math
import random

class Game(object):
    pass

class Status(object):
    def __init__(self, game):
        self.game = game
        self.happy = pygame.image.load('images/happy.png').convert_alpha()
        self.hungry = pygame.image.load('images/hungry.png').convert_alpha()

    def draw(self, surface):
        food_x = 150

        eaten = 0
        for food in self.game.level.foods:
            if food.state == 'eaten':
                eaten += 1

        for x in range(self.game.level.required_food-eaten):
            surface.blit(food.surface, (food_x, 25))
            food_x += 50

        if self.game.girlf.state == 'hungry':
            surface.blit(self.hungry, (20, 4))
        elif self.game.girlf.state == 'eating':
            surface.blit(self.happy, (25, 4))

class PlayArea(object):
    def __init__(self, game):
        self.surface = pygame.image.load('images/backdrop.png').convert()
        self.game = game

        self.blanket = Blanket()

    def get_rect(self):
        return pygame.Rect(0, 100, 800, 500)

    def draw(self, surface):
        surface.blit(self.surface, (0,0))
        self.blanket.draw(surface)
        self.game.girlf.draw(surface)

class Girlfriend(object):
    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'hungry'
        self.countdown = 4000

    def tick(self):
        if self.state == 'eating':
            self.countdown -= 20
            if self.countdown == 0:
                self.state = 'hungry'
                self.countdown = 4000

    def feed(self):
        self.state = 'eating'
        self.countdown = 4000

    def draw(self, surface):
        pass

    def get_rect(self):
        return pygame.Rect(20, 4, 93, 93)

class Blanket(object):
    def __init__(self):
        self.surface = pygame.image.load('images/blanket.png').convert_alpha()

    def draw(self, surface):
        surface.blit(self.surface, (248, 248))

class Hand(object):
    def __init__(self, game):
        self.x, self.y = 0, 0

        self.grabbing = False
        self.grabbed = None
        self.grab_offset = (0, 0)

        self.game = game

        self.open = pygame.image.load('images/openhand.png').convert_alpha()
        self.closed = pygame.image.load('images/closedhand.png').convert_alpha()

    def draw(self, surface, pos):
        if self.grabbed is not None:
            self.grabbed.x = self.x + self.grab_offset[0]
            self.grabbed.y = self.y + self.grab_offset[1]
            self.grabbed.draw(surface)

        if self.grabbing:
            surface.blit(self.closed, pos)
        else:
            surface.blit(self.open, pos)

        self.x, self.y = pos

    def get_draw_rect(self):
        if self.grabbing:
            return self.closed.get_rect()
        else:
            return self.open.get_rect()

    def grab(self):
        self.grabbing = True
        this = self.open.get_rect(top=self.y, left=self.x)

        for food in self.game.level.foods:
            if food.state in ['eaten', 'stolen']:
                continue
            if this.colliderect(food.get_rect()):
                food.state = 'hand_has'
                self.grabbed = food
                self.grab_offset = (food.x - self.x, food.y - self.y)
                return
    
    def release(self):
        self.grabbing = False
        if self.grabbed is not None:
            if (self.game.girlf.get_rect().collidepoint(self.x, self.y)
                and self.game.girlf.state == 'hungry'):
                    self.game.girlf.feed()
                    self.grabbed.state = 'eaten'
            elif not self.game.playarea.get_rect().collidepoint(self.x, self.y):
                self.grabbed.y = 100 + 20
                self.grabbed.state = 'available'
            else:
                self.grabbed.state = 'available'

            self.grabbed = None

class Food(object):
    def __init__(self, level, type):
        self.level = level

        self.x = 0
        self.y = 0

        self.x_round = 0
        self.y_round = 0

        self.ant_forces = []

        # hand_has, available, eaten, stolen 
        self.state = 'available'

        self.surface = pygame.image.load('images/%s.png'%type).convert_alpha()

    def get_rect(self):
        return self.surface.get_rect(left=self.x, top=self.y)

    def get_centre(self):
        return (self.x_round+15, self.y_round+20)

    def draw(self, surface):
        if self.state in ['available', 'hand_has']:
            pos = (self.x_round, self.y_round)
            surface.blit(self.surface, pos)

    def move(self, game):
        if self.state in ['eaten', 'stolen']:
            return

        for anthill in game.level.anthills:
            if anthill.get_hole().colliderect(self.get_rect()) and self.state != 'hand_has':
                self.state = 'stolen'
                return

        if len(self.ant_forces) > 0:
            self.x += self.ant_forces[0][0]
            self.y += self.ant_forces[0][1]

        self.x_round = int(round(self.x))
        self.y_round = int(round(self.y))

        self.ant_forces = []

class Ant(object):
    surface1 = None 
    surface2 = None

    def __init__(self, game, anthill):
        self.x = 0
        self.y = 0

        self.x_round = 0
        self.y_round = 0

        self.anthill = anthill
        self.game = game

        self.anim = 1
        self.move_time = 0

        self.angle = random.randint(0, 359)

        if self.surface1 is None:
            self.surface1 = pygame.image.load('images/ant_1.png').convert_alpha()
            self.surface2 = pygame.image.load('images/ant_2.png').convert_alpha()

    def get_rect(self):
        return pygame.Rect(self.x, self.y, 6, 8)

    def draw(self, surface):
        if self.anim == 1:
            surface.blit(self.surface1, (self.x_round, self.y_round))
        else:
            surface.blit(self.surface2, (self.x_round, self.y_round))

        self.anim *= -1

    def next_move(self):
        close_food_vects = []
        in_food = False

        # scan for food
        for food in self.game.level.foods:
            if food.state in ['hand_has', 'eaten', 'stolen']:
                continue
            elif len(food.ant_forces) > 5:
                continue
            elif self.get_rect().colliderect(food.get_rect()):
                dx = self.anthill.get_centre()[0] - self.x
                dy = self.anthill.get_centre()[1] - self.y
                in_food = True
            else:
                dx = food.get_centre()[0] - self.x
                dy = food.get_centre()[1] - self.y
        
            vl = math.sqrt(abs(dx)**2 + abs(dy)**2)

            if dx != 0:
                dx = (dx/vl)
            if dy != 0:
                dy = (dy/vl)

            if in_food:
                food.ant_forces.append((dx, dy))
                break
            elif vl < 100:
                close_food_vects.append((dx, dy, vl))

        if in_food:
            pass
        elif len(close_food_vects) > 0:
            close_food_vects.sort(key=lambda x: x[2])
            dx, dy, dummy = close_food_vects[0]
        else:
            if self.move_time == 0:
                self.angle = random.randint(min((self.angle-30)%360, (self.angle+30)%360), max((self.angle-30)%360, (self.angle+30)%360))
                self.move_time = 100
            else:
                self.move_time -= 1

            step = self.game.level.ant_speed 
            dx = step * math.cos(math.radians(self.angle))
            dy = step * math.sin(math.radians(self.angle))
            
        self.x += dx
        self.y += dy

        if self.y <= 100 or self.y >= 595: 
            self.angle = (360 - self.angle)
            self.move_time = 0

            if self.y <= 100: self.y = 101
            if self.y >= 595: self.y = 594

        if self.x <= 5 or self.x >= 795:
            self.angle = (180 - self.angle) % 360
            self.move_time = 0

            if self.x <=5: self.x = 6
            if self.x >=795: self.y = 594

        self.x_round = int(round(self.x))
        self.y_round = int(round(self.y))
            
class AntHill(object):
    def __init__(self, level):
        self.x = 100
        self.y = 100 

        self.max_ants = 1
        self.ants = []

        self.surface = pygame.image.load('images/anthill.png').convert_alpha()
        self.level = level

    def draw(self, surface):
        surface.blit(self.surface, (self.x, self.y))
        for ant in self.ants:
            ant.draw(surface)

    def ants_clear(self):
        for ant in self.ants:
            if self.get_hole().colliderect(ant.get_rect()):
                return False
        
        return True

    def get_rect(self):
        return self.surface.get_rect(top=self.y, left=self.x)

    def get_hole(self):
        return pygame.Rect(self.x+26, self.y+29, 19, 18)

    def get_centre(self):
        return (self.x+32, self.y+35)

    def drive_ants(self):
        if len(self.ants) < self.max_ants and self.ants_clear():
            # deploy new ant
            ant = Ant(self.level.game, self)
            ant.x, ant.y = ant.x_round, ant.y_round = self.get_centre()
            self.ants.append(ant)
        
        for ant in self.ants:
            ant.next_move()

def getNumberName(num):
    digits = ['first', 'second', 'third', 'fourth', 'fifth', 'sixth', 'seventh', 'eighth', 'ninth', 'tenth']
    teens = ['tenth', 'eleventh', 'twelfth', 'thirteenth', 'fourteenth', 'fifteenth', 'sixteenth', 'seventeenth', 'eighteenth', 'nineteenth']
    round_tens = ['tenth', 'twentieth', 'thirtieth', 'fortieth', 'fiftieth', 'sixtieth', 'seventieth', 'eightieth', 'ninetieth']
    tens = ['twenty', 'thirty', 'forty', 'fifty', 'sixty', 'seventy', 'eighty', 'ninety']

    if num < 10:
       return digits[num-1]
    elif num < 20:
       return teens[num-10]
    elif num%10 == 0:
        return round_tens[(num/10)-1]
    else:
        return '%s-%s' % (tens[(num/10)-2], digits[(num%10)-1])

def interstitial(screen, game, msg):
    font1 = pygame.font.Font(None, 80)
    font2 = pygame.font.Font(None, 30)

    text1 = font1.render(msg, True, (0, 255, 0))
    text2 = font2.render('click anywhere to continue', True, (0, 255, 0))

    bg_size = text1.get_size()
    bg = pygame.Surface((bg_size[0]+60, bg_size[1]+100))
    bg.blit(text1, (30, 30))
    bg.blit(text2, ((bg_size[0]+60-text2.get_size()[0])/2, 120))
    pygame.draw.rect(bg, (0, 255, 0), (0, 0, bg_size[0]+60, bg_size[1]+100), 5)

    if True in pygame.mouse.get_pressed():
        mouse_level = 2
    else:
        mouse_level = 1

    while 1:
        for event in pygame.event.get():
            if event.type == MOUSEBUTTONUP:
                mouse_level -=1
                if mouse_level == 0:
                    return
            elif event.type == QUIT:
                pygame.event.post(event)
                return

        game.playarea.draw(screen)
        screen.blit(bg, ((800-bg_size[0]-60)/2, ((600-bg_size[1]-60)/2)))
        game.hand.draw(screen, pygame.mouse.get_pos())

        pygame.display.flip()

def main():
    pygame.init()
    screen = pygame.display.set_mode((800, 600))
    pygame.display.set_caption('ant panic!')
    
    game = Game()
    game.hand = Hand(game)
    game.girlf = Girlfriend()
    game.status = Status(game)
    game.playarea = PlayArea(game)

    pygame.mouse.set_visible(False)
    pygame.time.set_timer(USEREVENT+1, 20)

    interstitial(screen, game, 'ant panic!')

    interstitial(screen, game, '%s date!' % getNumberName(1)) 
    game.level = levels.Level(game)
    game.level.next_level()

    while 1:
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                return
            elif event.type == MOUSEBUTTONDOWN:
                game.hand.grab()
            elif event.type == MOUSEBUTTONUP:
                game.hand.release()
            elif event.type == USEREVENT+1:
                game.girlf.tick()
                for anthill in game.level.anthills:
                    anthill.drive_ants()
                for food in game.level.foods:
                    food.move(game)

                game.playarea.draw(screen)
                game.status.draw(screen)

                for item in game.level.anthills+game.level.foods:
                    item.draw(screen)
            
                mouse = pygame.mouse.get_pos()
                game.hand.draw(screen, mouse)
                pygame.display.flip()

        # check for win or lose
        food_left = 0
        food_eaten = 0

        for food in game.level.foods:
            if food.state not in ['eaten', 'stolen']:
                food_left += 1
            if food.state == 'eaten':
                food_eaten += 1 

        if food_eaten == game.level.required_food:
            game.hand.grabbing = False
            game.hand.grabbed = None
            interstitial(screen, game, 'she loves you!')
            game.level.next_level()
            game.girlf.reset()
            interstitial(screen, game, '%s date!' % getNumberName(game.level.level_num)) 
        elif food_left < game.level.required_food - food_eaten:
            game.hand.grabbing = False
            game.hand.grabbed = None
            interstitial(screen, game, 'she loves you not!')
            game.level = levels.Level(game)
            game.level.next_level()
            game.girlf.reset()
            interstitial(screen, game, 'ant panic!')
            interstitial(screen, game, '%s date!' % getNumberName(1)) 

if __name__ == '__main__':
    main()
